unit rmSliderGauge;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TOrientation = (voVertical, voHorizontal);
  TBorder3DStyle = (bsNone, bsSingle, bsRaised, bsLowered);
  TActiveZoneStart = (asMinValue, asZero, asStartValue);

  TrmSliderGauge = class(TGraphicControl)
  private
    { Dclarations prives }
    FBmpBuffer: TBitmap;
    MouseCapt: boolean;
    FBGColor: TColor;
    FBorderColor: TColor;
    FActiveColor: TColor;
    FActiveZone3D: boolean;
    FActiveZoneStart: TActiveZoneStart;
    FActiveZoneStartValue: integer;
    FInactiveColor: TColor;
    FDisabledColor: TColor;
    FBorder3DStyle: TBorder3DStyle;
    FOrientation: TOrientation;
    FBorderMargin: integer;
    FButtonLength,
    FButtonBorderOffset: integer;
    FButtonColor: TColor;
    FButtonBorderColor: TColor;
    FButton3DStyle: TBorder3DStyle;
    FButtonNotch: boolean;
    FButtonNotchWidth: integer;
    FButtonNotchColor: TColor;
    FMinValue: integer;
    FMaxValue: integer;
    FPosition: integer;
    FOnChange: TNotifyEvent;
    procedure SetBGColor(value: TColor);
    procedure SetBorderColor(value: TColor);
    procedure SetActiveColor(value: TColor);
    procedure SetActiveZone3D(value: boolean);
    procedure SetActiveZoneStart(value: TActiveZoneStart);
    procedure SetActiveZoneStartValue(value: integer);
    procedure SetInactiveColor(value: TColor);
    procedure SetDisabledColor(value: TColor);
    procedure SetBorder3DStyle(value: TBorder3DStyle);
    procedure SetOrientation(value: TOrientation);
    procedure SetBorderMargin(value: integer);
    procedure SetButton3DStyle(value: TBorder3DStyle);
    procedure SetButtonLength(value: integer);
    procedure SetButtonBorderOffset(value: integer);
    procedure SetButtonColor(value: TColor);
    procedure SetButtonBorderColor(value: TColor);
    procedure SetButtonNotch(value: boolean);
    procedure SetButtonNotchColor(value: TColor);
    procedure SetButtonNotchWidth(value: integer);
    procedure SetMinValue(value: integer);
    procedure SetMaxValue(value: integer);
    procedure SetPosition(value: integer);
    procedure GetPosition(X, Y: integer);
  protected
    { Dclarations protges }
    procedure Paint; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure UpdateBmpBuffer;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
  public
    { Dclarations publiques }
    constructor Create(anOwner:TComponent); override;
    destructor Destroy; override;
    procedure Free;
  published
    { Dclarations publies }
    property BGColor: TColor read FBGColor write SetBGColor;
    property BorderColor: TColor read FBorderColor write SetBorderColor;
    property ActiveColor: TColor read FActiveColor write SetActiveColor;
    property ActiveZone3D: boolean read FActiveZone3D write SetActiveZone3D;
    property ActiveZoneStart: TActiveZoneStart read FActiveZoneStart write SetActiveZoneStart;
    property ActiveZoneStartValue: integer read FActiveZoneStartValue write SetActiveZoneStartValue;
    property InactiveColor: TColor read FInactiveColor write SetInactiveColor;
    property DisabledColor: TColor read FDisabledColor write SetDisabledColor;
    property Border3DStyle: TBorder3DStyle read FBorder3DStyle write SetBorder3DStyle;
    property Orientation: TOrientation read FOrientation write SetOrientation;
    property BorderMargin: integer read FBorderMargin write SetBorderMargin;
    property Button3DStyle: TBorder3DStyle read FButton3DStyle write SetButton3DStyle;
    property ButtonLength: integer read FButtonLength write SetButtonLength;
    property ButtonBorderOffset: integer read FButtonBorderOffset write SetButtonBorderOffset;
    property ButtonColor: TColor read FButtonColor write SetButtonColor;
    property ButtonBorderColor: TColor read FButtonBorderColor write SetButtonBorderColor;
    property ButtonNotch: boolean read FButtonNotch write SetButtonNotch;
    property ButtonNotchColor: TColor read FButtonNotchColor write SetButtonNotchColor;
    property ButtonNotchWidth: integer read FButtonNotchWidth write SetButtonNotchWidth;
    property MinValue: integer read FMinValue write SetMinValue;
    property MaxValue: integer read FMaxValue write SetMaxValue;
    property Position: integer read FPosition write SetPosition;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    { Standard events }
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    { Standard properties }
    property Align;
    property Enabled;
    property Height;
    property Width;
    property PopupMenu;
    property Hint;
    property ShowHint;
    property Tag;
    property Visible;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('OptoElec', [TrmSliderGauge]);
end;

constructor TrmSliderGauge.Create(anOwner:TComponent);
begin
  Inherited Create(anOwner);
  FBmpBuffer := TBitmap.Create;
  Height := 15;
  Width := 193;
  MouseCapture := false;
  FBGColor := clGray;
  FBorderColor := clAqua;
  FActiveColor := clAqua;
  FActiveZone3D := true;
  FActiveZoneStart := asMinValue;
  FActiveZoneStartValue := 10;
  FInactiveColor := clGray;
  FDisabledColor := clSilver;
  FOrientation := voHorizontal;
  FBorder3DStyle := bsLowered;
  FBorderMargin := 3;
  FButton3DStyle := bsRaised;
  FButtonLength := 14;
  FButtonBorderOffset := 1;
  FButtonColor := clFuchsia;
  FButtonBorderColor := clGray;
  FButtonNotch := false;
  FButtonNotchWidth := 2;
  FButtonNotchColor := clAqua;
  FMinValue := 0;
  FMaxValue := 100;
  FPosition := 30;
end;

destructor TrmSliderGauge.Destroy;
begin
  If FBmpBuffer <> NIL then FBmpBuffer.free;
  Inherited Destroy;
end;

procedure TrmSliderGauge.Free;
begin
  FBmpBuffer.Free;
  Inherited Free;
end;

procedure TrmSliderGauge.SetBounds(aLeft, aTop, aWidth, aHeight: Integer);
begin
  Inherited ;
  If FBmpBuffer <> NIL then begin
    FBmpBuffer.Destroy;
    FBmpBuffer := TBitmap.Create;
    FBmpBuffer.Height := aHeight;
    FBmpBuffer.Width := aWidth;
  end;
end;

procedure TrmSliderGauge.UpdateBmpBuffer;

  procedure DrawBackground;
  begin
    With FBmpBuffer.Canvas do begin
      if enabled then begin
        Brush.Color := FBGColor;
        if FBorder3DStyle = bsNone then
          Pen.Color := FBGColor
        else
          Pen.Color := FBorderColor;
      end else begin
        brush.Color := FDisabledColor;
        Pen.Color := FDisabledColor;
      end;
      Rectangle(0, 0, width, height);
    end;
  end;

  procedure DrawInactiveZone;
  var
    X1, Y1, X2, Y2: integer;
  begin
    if enabled and (FInactiveColor = FBGColor) then exit;
    With FBmpBuffer.Canvas do begin
      if enabled then begin
        Pen.Color := FInactiveColor;
        brush.Color := FInactiveColor;
      end else begin
        Pen.Color := FDisabledColor;
        brush.Color := FDisabledColor;
      end;
      if (FOrientation = voHorizontal) then begin
        X1 := FBorderMargin;
        Y1 := FBorderMargin;
        X2 := width - FBorderMargin;
        Y2 := height - FBorderMargin ;
      end else if (FOrientation = voVertical) then begin
        X1 := FBorderMargin;
        Y1 := FBorderMargin;
        X2 := width - FBorderMargin ;
        Y2 := height - FBorderMargin;
      end;
      rectangle(X1, Y1, X2, Y2);
    end;
  end;

  function RatioPosition: real;
  begin
    try
      if ((FMaxValue - FMinValue) <> 0) and ((FPosition - FMinValue) <> 0) then
        RatioPosition := trunc((100 / ((FMaxValue - FMinValue) / (FPosition - FMinValue))))
      else
        RatioPosition := 1;
    except
      RatioPosition := 1;
    end;
  end;

  function RatioStartValue: real;
  begin
    try
      if ((FMaxValue - FMinValue) <> 0) and ((FActiveZoneStartValue - FMinValue) <> 0) then
        RatioStartValue := trunc((100 / ((FMaxValue - FMinValue) / (FActiveZoneStartValue - FMinValue))))
      else
        RatioStartValue := 1;
    except
      RatioStartValue := 1;
    end;
  end;

  function RatioZero: real;
  begin
    try
      if ((FMaxValue - FMinValue) <> 0) and (FMinValue <> 0) then
        RatioZero := trunc((100 / ((FMaxValue - FMinValue) / (0 - FMinValue))))
      else
        RatioZero := 1;
    except
      RatioZero := 1;
    end;
  end;

  procedure DrawActiveZone;
  var
    Z0, Z1, Z2, LengthReallyUsed,
    X1, Y1, X2, Y2: integer;
  begin
    With FBmpBuffer.Canvas do begin
      if enabled then begin
        Pen.Color := FActiveColor;
        brush.Color := FActiveColor;
      end else begin
        Pen.Color := FDisabledColor;
        brush.Color := FDisabledColor;
      end;
      if (FOrientation = voHorizontal) then begin
        LengthReallyUsed := width - (2 * FBorderMargin);
        Z0 := FBorderMargin + trunc(LengthReallyUsed / 100 * RatioZero);
        Z1 := FBorderMargin + trunc(LengthReallyUsed / 100 * RatioStartValue);
        Z2 := FBorderMargin + trunc(LengthReallyUsed / 100 * RatioPosition);
        if (FActiveZoneStart = asMinValue) then begin
          X1 := FBorderMargin;
          Y1 := FBorderMargin;
          X2 := Z2;
          Y2 := height - FBorderMargin ;
        end else if (FActiveZoneStart = asZero) then begin
          X1 := Z0;
          Y1 := FBorderMargin;
          X2 := Z2;
          Y2 := height - FBorderMargin ;
        end else if (FActiveZoneStart = asStartValue) then begin
          X1 := Z1;
          Y1 := FBorderMargin;
          X2 := Z2;
          Y2 := height - FBorderMargin ;
        end;
      end else if (FOrientation = voVertical) then begin
        LengthReallyUsed := height - (2 * FBorderMargin);
        Z0 := height - FBorderMargin - trunc(LengthReallyUsed / 100 * RatioZero);
        Z1 := height - FBorderMargin - trunc(LengthReallyUsed / 100 * RatioStartValue);
        Z2 := height - FBorderMargin - trunc(LengthReallyUsed / 100 * RatioPosition);
        if (FActiveZoneStart = asMinValue) then begin
          X1 := FBorderMargin;
          Y1 := Z2;
          X2 := width - FBorderMargin ;
          Y2 := height - FBorderMargin;
        end else if (FActiveZoneStart = asZero) then begin
          X1 := FBorderMargin;
          Y1 := Z2;
          X2 := width - FBorderMargin ;
          Y2 := Z0;
        end else if (FActiveZoneStart = asStartValue) then begin
          X1 := FBorderMargin;
          Y1 := Z2;
          X2 := width - FBorderMargin ;
          Y2 := Z1;
        end;
      end;
      rectangle(X1, Y1, X2, Y2);

      // 3D effect ?
      if FActiveZone3D then begin
        Pen.Color := clWhite;
        moveTo(X1, Y2 - 1);
        LineTo(X1, Y1);
        LineTo(X2 - 1, Y1);
        Pen.Color := clGray;
        LineTo(X2 - 1, Y2 - 1);
        LineTo(X1, Y2 - 1);
      end;
    end;
  end;

  procedure Draw3DBorder;
  begin
    if (FBorder3DStyle = bsNone) or (FBorder3DStyle = bsSingle) then exit;
    With FBmpBuffer.Canvas do begin
      if FBorder3DStyle <> bsNone then begin
        if FBorder3DStyle = bsLowered then
          Pen.Color := clGray
        else
          Pen.Color := clWhite;
        moveTo(0, 0);
        LineTo(0, height - 1);
        moveTo(0, 0);
        LineTo(width - 1, 0);
        if FBorder3DStyle = bsLowered then
          Pen.Color := clWhite
        else
          Pen.Color := clGray;
        moveTo(0, height - 1);
        LineTo(width - 1, height - 1);
        moveTo(width - 1, height - 1);
        LineTo(width - 1, 0);
      end;
    end;
  end;

  procedure DrawButton;
  var
    ButtonCenterPos, LengthReallyUsed,
    X1, Y1, X2, Y2: integer;
  begin
    With FBmpBuffer.Canvas do begin
      if enabled then begin
        brush.Color := FButtonColor;
      end else begin
        brush.Color := FDisabledColor;
      end;
      if (FButton3DStyle = bsNone) then
        Pen.Color := FButtonColor
      else
        Pen.Color := FButtonBorderColor;

      if (FOrientation = voHorizontal) then begin
        LengthReallyUsed := width - FButtonLength - 2;
        // ButtonCenterPos := trunc((LengthReallyUsed / 100) * FPosition) + trunc(FButtonLength / 2) + 1;
        ButtonCenterPos := trunc(LengthReallyUsed / 100 * RatioPosition) + trunc(FButtonLength / 2) + 1;
        X1 := ButtonCenterPos - trunc(FButtonLength / 2);
        Y1 := FButtonBorderOffset;
        X2 := ButtonCenterPos + trunc(FButtonLength / 2);
        Y2 := height - FButtonBorderOffset ;
      end else if (FOrientation = voVertical) then begin
        LengthReallyUsed := height - FButtonLength - 2;
        // ButtonCenterPos := trunc((LengthReallyUsed / 100) * (100 - FPosition)) + trunc(FButtonLength / 2) + 1;
        ButtonCenterPos := height - trunc(LengthReallyUsed / 100 * RatioPosition) - trunc(FButtonLength / 2) - 1;
        X1 := FButtonBorderOffset;
        Y1 := ButtonCenterPos - trunc(FButtonLength / 2);
        X2 := width - FButtonBorderOffset;
        Y2 := ButtonCenterPos + trunc(FButtonLength / 2);
      end;
      rectangle(X1, Y1, X2, Y2);
    end;
  end;

  procedure DrawButton3DBorder;
  var
    ButtonCenterPos, LengthReallyUsed,
    X1, Y1, X2, Y2: integer;
  begin
    if (FButton3DStyle = bsNone) or (FButton3DStyle = bsSingle) then exit;
    With FBmpBuffer.Canvas do begin
      if (FOrientation = voHorizontal) then begin
        LengthReallyUsed := width - FButtonLength - 2;
        // ButtonCenterPos := trunc((LengthReallyUsed / 100) * FPosition) + trunc(FButtonLength / 2) + 1;
        ButtonCenterPos := trunc(LengthReallyUsed / 100 * RatioPosition) + trunc(FButtonLength / 2) + 1;
        X1 := ButtonCenterPos - trunc(FButtonLength / 2);
        Y1 := FButtonBorderOffset;
        X2 := ButtonCenterPos + trunc(FButtonLength / 2);
        Y2 := height - FButtonBorderOffset ;
      end else if (FOrientation = voVertical) then begin
        LengthReallyUsed := height - FButtonLength - 2;
        // ButtonCenterPos := trunc((LengthReallyUsed / 100) * (100 - FPosition)) + trunc(FButtonLength / 2) + 1;
        ButtonCenterPos := height - trunc(LengthReallyUsed / 100 * RatioPosition) - trunc(FButtonLength / 2) - 1;
        X1 := FButtonBorderOffset;
        Y1 := ButtonCenterPos - trunc(FButtonLength / 2);
        X2 := width - FButtonBorderOffset;
        Y2 := ButtonCenterPos + trunc(FButtonLength / 2);
      end;

      if (FButton3DStyle = bsLowered) then
        Pen.Color := clGray
      else
        Pen.Color := clWhite;
      moveTo(X1, Y1);
      LineTo(X1, Y2 - 1);
      moveTo(X1, Y1);
      LineTo(X2, Y1);
      if (FButton3DStyle = bsLowered) then
        Pen.Color := clWhite
      else
         Pen.Color := clGray;
      moveTo(X1, Y2 - 1);
      LineTo(X2 - 1, Y2 - 1);
      moveTo(X2 - 1, Y2 - 1);
      LineTo(X2 - 1, Y1);
    end;
  end;

  procedure DrawButtonNotch;
  var
    ButtonCenterPos, LengthReallyUsed,
    X1, Y1, X2, Y2: integer;
  begin
    if (not FButtonNotch) or (not enabled) then exit;
    With FBmpBuffer.Canvas do begin
      brush.Color := FButtonNotchColor;
      Pen.Color := FButtonNotchColor;
      if (FOrientation = voHorizontal) then begin
        LengthReallyUsed := width - FButtonLength - 2;
        // ButtonCenterPos := trunc((LengthReallyUsed / 100) * FPosition) + trunc(FButtonLength / 2) + 1;
        ButtonCenterPos := trunc(LengthReallyUsed / 100 * RatioPosition) + trunc(FButtonLength / 2) + 1;
        X1 := ButtonCenterPos - trunc(FButtonNotchWidth / 2);
        Y1 := FButtonBorderOffset + 2;
        X2 := ButtonCenterPos + trunc(FButtonNotchWidth / 2);
        Y2 := height - FButtonBorderOffset - 2;
      end else if (FOrientation = voVertical) then begin
        LengthReallyUsed := height - FButtonLength - 2;
        // ButtonCenterPos := trunc((LengthReallyUsed / 100) * (100 - FPosition)) + trunc(FButtonLength / 2) + 1;
        ButtonCenterPos := height - trunc(LengthReallyUsed / 100 * RatioPosition) - trunc(FButtonLength / 2) - 1;
        X1 := FButtonBorderOffset + 2;
        Y1 := ButtonCenterPos - trunc(FButtonNotchWidth / 2);
        X2 := width - FButtonBorderOffset - 2;
        Y2 := ButtonCenterPos + trunc(FButtonNotchWidth / 2);
      end;
      rectangle(X1, Y1, X2, Y2);
    end;
  end;


var
  i, X1, Y1, X2, Y2: integer;
  TempSize1, TempSize2: integer;
begin
  DrawBackground;
  DrawInactiveZone;
  DrawActiveZone;
  Draw3DBorder;
  DrawButton;
  DrawButton3DBorder;
  DrawButtonNotch;
end;

procedure TrmSliderGauge.Paint;
begin
  UpdateBmpBuffer;
  Canvas.Draw(0, 0, FBmpBuffer);
end;

procedure TrmSliderGauge.SetBGColor(value: TColor);
begin
  if value <> FBGColor then
  begin
    FBGColor := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetBorderColor(value: TColor);
begin
  if value <> FBorderColor then
  begin
    FBorderColor := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetActiveZone3D(value: boolean);
begin
  if value <> FActiveZone3D then
  begin
    FActiveZone3D := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetActiveZoneStart(value: TActiveZoneStart);
begin
  if value <> FActiveZoneStart then
  begin
    FActiveZoneStart := value;
    if (FActiveZoneStart = asZero) then begin
      if ((FMinValue > 0) and (FMaxValue > 0)) or ((FMinValue < 0) and (FMaxValue < 0)) then begin
        FMinValue := -50;
        FMaxValue := 50;
      end;
    end;
    if (FActiveZoneStart = asStartValue) then begin
      if not ((FActiveZoneStartValue > FMinValue) and (FActiveZoneStartValue < FMaxValue))then begin
        FActiveZoneStartValue := FMinValue;
      end;
    end;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetActiveZoneStartValue(value: integer);
begin
  if value <> FActiveZoneStartValue then
  begin
    FActiveZoneStartValue := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetActiveColor(value: TColor);
begin
  if value <> FActiveColor then
  begin
    FActiveColor := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetInactiveColor(value: TColor);
begin
  if value <> FInactiveColor then
  begin
    FInactiveColor := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetDisabledColor(value: TColor);
begin
  if value <> FDisabledColor then
  begin
    FDisabledColor := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetBorder3DStyle(value: TBorder3DStyle);
begin
  if value <> FBorder3DStyle then
  begin
    FBorder3DStyle := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetOrientation(value: TOrientation);
var
  oldWidthValue: integer;
begin
  if value <> FOrientation then
  begin
    FOrientation := value;
    if (csDesigning in ComponentState) and not(csReading in ComponentState) then begin
      oldWidthValue := width;
      width := height;
      height := oldWidthValue;
    end;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetBorderMargin(value: integer);
begin
  if value <> FBorderMargin then
  begin
    FBorderMargin := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetButton3DStyle(value: TBorder3DStyle);
begin
  if value <> FButton3DStyle then
  begin
    FButton3DStyle := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetButtonLength(value: integer);
begin
  if value <> FButtonLength then
  begin
    FButtonLength := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetButtonBorderOffset(value: integer);
begin
  if value <> FButtonBorderOffset then
  begin
    FButtonBorderOffset := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetButtonColor(value: TColor);
begin
  if value <> FButtonColor then
  begin
    FButtonColor := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetButtonBorderColor(value: TColor);
begin
  if value <> FButtonBorderColor then
  begin
    FButtonBorderColor := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetButtonNotch(value: boolean);
begin
  if value <> FButtonNotch then
  begin
    FButtonNotch := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetButtonNotchColor(value: TColor);
begin
  if value <> FButtonNotchColor then
  begin
    FButtonNotchColor := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.SetButtonNotchWidth(value: integer);
begin
  if value <> FButtonNotchWidth then
  begin
    FButtonNotchWidth := value;
    Paint;
  end;
end;

procedure TrmSliderGauge.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
    GetPosition(X, Y);
    Paint;
    MouseCapt := true;
  end;
  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TrmSliderGauge.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
    MouseCapt := false;
  end;
  inherited MouseUp(Button, Shift, X, Y);
end;

procedure TrmSliderGauge.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if MouseCapt then
  begin
    if (X < 0) or (X > width) or (Y < 0) or (Y > height) then exit;
    GetPosition(X, Y);
    Paint;
  end;
  inherited MouseMove (shift, X, Y);
end;

procedure TrmSliderGauge.GetPosition(X, Y: integer);

  function GetRatio(iLen: integer): real;
  begin
    try
      Result := ((FMaxValue - FMinValue) / iLen);
    except
      Result := 1;
    end;
  end;

var
  ratio: real;
  i: integer;
  LengthReallyUsed: integer;
begin
  i := 0;
  if (FOrientation = voHorizontal) then begin
    LengthReallyUsed := width - FButtonLength - 2;
    ratio := GetRatio(LengthReallyUsed);
    try
      if X < trunc(FButtonLength / 2) then
        i := FMinValue
      else if X > (width - trunc(FButtonLength / 2)) then
        i := FMaxValue
      else
         i := trunc((X - trunc(FButtonLength / 2)) * ratio) + FMinValue;
    except
      i := FMinValue;
    end;
  end else if (FOrientation = voVertical) then begin
    LengthReallyUsed := height - FButtonLength - 2;
    ratio := GetRatio(LengthReallyUsed);
    try
      if Y > height - trunc(FButtonLength / 2) then
        i := FMinValue
      else if Y < trunc(FButtonLength / 2) then
        i := FMaxValue
      else
        // i := trunc((height - Y) * ratio) + FMinValue;
         i := trunc(((height - Y) - trunc(FButtonLength / 2)) * ratio) + FMinValue;
    except
      i := FMinValue;
    end;
  end;
  if i < FMinValue then i := FMinValue;
  if i > FMaxValue then i := FMaxValue;
  SetPosition(i);
end;

procedure TrmSliderGauge.SetMinValue(value: integer);
begin
  if value <> FMinValue then
  begin
    FMinValue := value;
    if FMinValue > FMaxValue then begin
      FMinValue := 0;
      FMaxValue := 100;
    end;
    if FPosition < FMinValue then
      SetPosition(FMinValue);
    Paint;
  end;
end;

procedure TrmSliderGauge.SetMaxValue(value: integer);
begin
  if value <> FMaxValue then
  begin
    FMaxValue := value;
    if FMaxValue < FMinValue then begin
      FMinValue := 0;
      FMaxValue := 100;
    end;
    if FPosition > FMaxValue then
      SetPosition(FMaxValue);
    Paint;
  end;
end;

procedure TrmSliderGauge.SetPosition(value: integer);
begin
  if value <> FPosition then
  begin
    if value > FMaxValue then
      FPosition := FMaxValue
    else if value < FMinValue then
      FPosition := FMinValue
    else
      FPosition := value;
    Paint;
    if assigned(FOnChange) then
      FOnChange(self);
  end;
end;


end.
